<?php
class CSVFileIterator implements Iterator {

    private $paths;
    /** @var file resource */
    private $current = false;
    private $position = 0;

    public function __construct($sortedPathsArray) {
        if (!is_array($sortedPathsArray)) {
            throw new InvalidArgumentException("Given variable is not a paths array.");
        }
        $this->paths = $sortedPathsArray;
    }

    /**
     * This is the destructor. It close the CSV file.
     *
     * @access public
     */
    public function __destruct() {
        if (isset($this->current) && $this->current !== false) {
            fclose($this->current);
        }
        $this->current = false;
        //unset($this->current);
    }

    /**
     * This method move the file pointer to the next row.
     *
     * @access public
     */
    public function next() {
        $this->position++;
        $this->open();
    }

    /**
     * This method reset the file handler.
     *
     * @access public
     */
    public function rewind() {
        $this->position = 0;
        $this->open();
    }

    /**
     * This method returns the current path.
     *
     * @access public
     */
    public function key() {
        return $this->paths[$this->position];
    }

    /**
     * This methods return the current file pointer.
     *
     * @access public
     * @return pointer
     */
    public function current() {
        return $this->current;
    }

    /**
     * This method checks if the current file is readable.
     *
     * @access public
     * @return boolean If the current row is readable.
     */
    public function valid() {
        if ($this->current === false) {
            return false;
        }
        return true;
    }

    private function open() {
        if (count($this->paths) < ($this->position + 1)) {
            $this->current = false;
            return;
        }
        $newPath = $this->paths[$this->position];
        if ($this->valid()) {
            fclose($this->current);
        }

        if(!file_exists($newPath)) {
            $this->position++;
            while(count($this->paths) >= ($this->position + 1)) {
                $newPath = $this->paths[$this->position];
                if(file_exists($newPath)) {
                    $this->current = fopen($newPath, "r");
                    return;
                }
                $this->position++;
            }
            $this->current = false;
            return;
        }
        $this->current = fopen($newPath, "r");
    }
}

?>
